@@ -105,12 +105,11 @@ def _get_os_environ(envvar_prefix=None):
     for k in environkeys:
         newkey = k.split('_', 1)[-1].lower()
         ret[newkey] = os.environ[k]
-    # If the only environ key is region name, don't make a cloud, because
-    # it's being used as a cloud selector
-    if not environkeys or (
-            len(environkeys) == 1 and 'region_name' in ret):
-        return None
-    return ret
+    # If the only environ keys are cloud and region_name, don't return anything
+    # because they are cloud selectors
+    if set(environkeys) - set(['OS_CLOUD', 'OS_REGION_NAME']):
+        return ret
+    return None
 
 
 def _merge_clouds(old_dict, new_dict):
@@ -174,8 +173,9 @@ class OpenStackConfig(object):
     def __init__(self, config_files=None, vendor_files=None,
                  override_defaults=None, force_ipv4=None,
                  envvar_prefix=None, secure_files=None,
-                 pw_func=None):
+                 pw_func=None, session_constructor=None):
         self.log = _log.setup_logging(__name__)
+        self._session_constructor = session_constructor
 
         self._config_files = config_files or CONFIG_FILES
         self._secure_files = secure_files or SECURE_FILES
@@ -388,7 +388,10 @@ class OpenStackConfig(object):
         if 'regions' in config:
             return self._expand_regions(config['regions'])
         elif 'region_name' in config:
-            regions = config['region_name'].split(',')
+            if isinstance(config['region_name'], list):
+                regions = config['region_name']
+            else:
+                regions = config['region_name'].split(',')
             if len(regions) > 1:
                 warnings.warn(
                     "Comma separated lists in region_name are deprecated."
@@ -637,7 +640,7 @@ class OpenStackConfig(object):
         # completely broken
         return cloud
 
-    def register_argparse_arguments(self, parser, argv, service_keys=[]):
+    def register_argparse_arguments(self, parser, argv, service_keys=None):
         """Register all of the common argparse options needed.
 
         Given an argparse parser, register the keystoneauth Session arguments,
@@ -656,6 +659,9 @@ class OpenStackConfig(object):
                                                     is requested
         """
 
+        if service_keys is None:
+            service_keys = []
+
         # Fix argv in place - mapping any keys with embedded _ in them to -
         _fix_argv(argv)
 
@@ -1124,7 +1130,8 @@ class OpenStackConfig(object):
             config=self._normalize_keys(config),
             force_ipv4=force_ipv4,
             auth_plugin=auth_plugin,
-            openstack_config=self
+            openstack_config=self,
+            session_constructor=self._session_constructor,
         )
 
     def get_one_cloud_osc(
